@@ -20,27 +20,59 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include <stdio.h>
-
 #include <Python.h>
 
+#include <stdio.h>
+
 extern PyTypeObject ToxCoreType;
 extern PyObject* ToxCoreError;
+extern PyMethodDef Tox_methods[];
 
+#if PY_MAJOR_VERSION >= 3
+struct PyModuleDef moduledef = {
+  PyModuleDef_HEAD_INIT,
+  "tox",
+  "Python Toxcore module",
+  -1,
+  Tox_methods,
+  NULL,
+  NULL,
+  NULL,
+  NULL
+};
+#endif
+
+#if PY_MAJOR_VERSION >= 3
+PyObject *PyInit_tox(void) {
+  PyObject *m = PyModule_Create(&moduledef);
+#else
 PyMODINIT_FUNC inittox(void) {
-  PyObject* m = Py_InitModule("tox", NULL);
+  PyObject *m = Py_InitModule("tox", NULL);
+#endif
+
   if (m == NULL) {
-    return;
+    goto error;
   }
 
   // Initialize tox.core
   if (PyType_Ready(&ToxCoreType) < 0) {
     fprintf(stderr, "Invalid PyTypeObject `ToxCoreType'\n");
-    return;
+    goto error;
   }
 
   Py_INCREF(&ToxCoreType);
   PyModule_AddObject(m, "Tox", (PyObject*)&ToxCoreType);
 
   ToxCoreError = PyErr_NewException("tox.OperationFailedError", NULL, NULL);
+
+#if PY_MAJOR_VERSION >= 3
+  return m;
+#endif
+
+error:
+#if PY_MAJOR_VERSION >= 3
+  return NULL;
+#else
+  return;
+#endif
 }
